-- ===================================================================

local reg_alias = ocutil.safe_register_alias
local reg_item  = ocutil.safe_register_item
local reg_node  = ocutil.safe_register_node

local needitem
local neednode

-- ===================================================================

needitem = "mobs:feather"
--
if minetest.registered_items [needitem] == nil then
    reg_item (needitem, {
        description = "Feather" ,
        inventory_image = "creatures_feather.png" ,
    })
end

-- ===================================================================

needitem = "mobs:snowball"
--
if ocutil.item_missing (needitem) then
    reg_item (needitem, {
        description     = "Snowball"             ,
        inventory_image = "default_snowball.png" ,
    })
end

-- ===================================================================

reg_alias ("snowball:snowball", "mobs:snowball")

-- ===================================================================

nodepart = "glowstone"
fromnode = "coderblocks:" .. nodepart
--
if ocutil.node_exists (fromnode) then
    stairs.register_all (nodepart, fromnode,
        { cracky=2                              } ,
        { "glowstone_glowstone.png"             } ,
        "Glowstone"                               ,
        stairs.snd_stone)
end

-- ===================================================================

neednode = "birthstones:opalblock"
--
if ocutil.thing_missing (neednode) then
    reg_node (neednode, {
        description = "Opal Block",
        tiles = { "birthstones_opal_block.png" } ,
        is_ground_content = true,
        groups = { cracky=1, level=3 } ,
        sounds = default.sound_stone(),
    })
end

-- ===================================================================

neednode = "caverealms:salt_crystal"
--
if ocutil.thing_missing (neednode) then
    reg_node (neednode, {
        description = "Salt Crystal" ,
        tiles = { "caverealms_salt_crystal.png" } ,
        is_ground_content = true,
        groups = {cracky=3},
        sounds = default.sound_glass() ,
        light_source = 11,
        paramtype = "light",
        use_texture_alpha = true,
        drawtype = "glasslike",
        sunlight_propagates = true,
    })
end

-- ===================================================================
-- A 2D but HQ planet.

if type (placecraft) == "table" then
    placecraft.register_item ("toys:planet", {
        description = "Planet"          ,
        drawtype    = "sprite"          ,
        image       = "toys_planet.png" ,
        vsize       = 1.00              ,
    })
end

-- ===================================================================
-- Some 2D but HQ gems.

local ngems = {
    "amethyst" , "aquamarine" , "diamond"  ,
    "emerald"  , "ruby"       , "sapphire" ,
}

-- ===================================================================

if type (placecraft) == "table" then
    for _, basename in ipairs (ngems) do
        local node = "ngems:" .. basename
        placecraft.register_item (node, {
            description = ocutil.first_to_upper (basename) ,
            drawtype    = "sprite"                         ,
            image       = "ngems_" .. basename .. ".png"   ,
            vsize       = 0.50                             ,
        })
    end
end

-- ===================================================================

minetest.register_node ("coderblocks:wishingirl", {
    description = "It's the block that isn't ad hoc" ,
    tiles       = {
        "wishingirl_painting_1.jpg", -- top
        "wishingirl_painting_1.jpg", -- bottom
        "wishingirl_painting_1.jpg", -- right
        "wishingirl_painting_1.jpg", -- left
        "wishingirl_painting_1.jpg", -- back
        "wishingirl_painting_1.jpg"  -- front
    } ,

    light_source   = 10                          ,
    paramtype      = "light"                     ,

    groups = { oddly_breakable_by_hand = 2 }     ,
    sounds = default.sound_stone() ,
})
                
-- ===================================================================

minetest.register_craftitem ("coderblocks:comb", {
    description = "Final MT Comb" ,
    inventory_image = "occomb.png",
    stack_max = 64,
    groups = { craftitem = 1 },
})

minetest.register_craftitem ("coderblocks:shampoo", {
    description = "Final MT shampoo" ,
    inventory_image = "ocshampoo.png",
    stack_max = 64,
    groups = { craftitem = 1 },
})

minetest.register_craftitem ("coderblocks:soap", {
    description = "Final MT bar soap" ,
    inventory_image = "ocsoap.png",
    stack_max = 64,
    groups = { craftitem = 1 },
})

minetest.register_craftitem ("coderblocks:gloreo", {
    description = "Gloreo Cookie",
    inventory_image = "gloreo.png",
    on_use = minetest.item_eat (4) ,
})

minetest.register_craftitem ("coderblocks:pacifier", {
    description = "Pacifier" ,
    inventory_image = "coderblocks_pacifier.png" ,
})

minetest.register_craftitem ("coderblocks:babybottle", {
    description = "Baby Bottle" ,
    inventory_image = "coderblocks_babybottle.png" ,
})

-- ===================================================================
-- End of file.
