-- ===================================================================

local reg_alias = ocutil.safe_register_alias

-- ===================================================================

-- These two need to be handled using aliases. Reason: They may be us-
-- ed by mapgen schems that haven't been updated yet to replace them.

reg_alias ("default:acacia_tree" , "default:acacia_trunk" )
reg_alias ("default:aspen_tree"  , "default:aspen_trunk"  )

-- ===================================================================

local cvtmap = {}

cvtmap ["default:chest_open"              ] =
        "default:chest"

cvtmap [ "default:junglewood_cube"        ] =
        "default:jungle_wood_cube"

cvtmap [ "default:junglewood_doublepanel" ] =
        "default:jungle_wood_doublepanel"

cvtmap [ "default:junglewood_halfstair"   ] =
        "default:jungle_wood_halfstair"

cvtmap [ "default:junglewood_micropanel"  ] =
        "default:jungle_wood_micropanel"

cvtmap [ "default:junglewood_microslab"   ] =
        "default:jungle_wood_microslab"

cvtmap [ "default:junglewood_panel"       ] =
        "default:jungle_wood_panel"

cvtmap [ "default:junglewood_thinstair"   ] =
        "default:jungle_wood_thinstair"

cvtmap ["stairs:stair_jungletree"         ] =
        "stairs:stair_jungle_wood"

cvtmap ["default:tree_outerstair"         ] =
        "stairs:stair_outer_wood"

cvtmap ["moontest:vacuum"                 ] =
        "default:vacuum"

cvtmap ["carpet:wool:Grey"                ] =
        "carpet:grey"

cvtmap ["maze:closer"                     ] =
        "default:cobble"

cvtmap ["default:grass_1"                 ] =
        "air"
cvtmap ["default:grass_2"                 ] =
        "air"

-- ===================================================================

local function register_cvtmap (old_node)

    if     ocutil.node_exists  (old_node) then
        ocutil.add_item_groups (old_node,
            { cvtmap=1, not_in_creative_inventory=1 })
    elseif ocutil.thing_exists (old_node) then
        return
    else
        minetest.register_node (":" .. old_node, {
            groups = { cvtmap=1, not_in_creative_inventory=1 } ,
        })
    end
end

-- ===================================================================

for old_node, _ in pairs (cvtmap) do
    register_cvtmap (old_node)
end

-- ===================================================================

default.convert_node = function (old_node, new_node)
    local rtn = "convert_node"

    if ocutil.str_empty (old_node) then ocutil.panic (rtn .. " 1") end
    if ocutil.str_empty (new_node) then ocutil.panic (rtn .. " 2") end

    if cvtmap [old_node] ~= nil    then ocutil.panic (rtn .. " 3") end
    if cvtmap [new_node] ~= nil    then ocutil.panic (rtn .. " 4") end

    if ocutil.node_missing (new_node) then return end

    cvtmap [old_node] = new_node
    register_cvtmap (old_node)
end

-- ===================================================================

minetest.register_lbm ({

    -- For LBM version:
    name  = ":default:cvtmap" ,
    label = ":default:cvtmap" ,
    run_at_every_load = true  ,

    -- For ABM version:
    catch_up = false ,
    chance   = 1     ,
    interval = 3     ,

    nodenames = { "group:cvtmap" } ,

    action = function (pos, node)
        local newname =  cvtmap [node.name]
        if    newname ~= nil and
            ocutil.node_exists (newname) then
            minetest.set_node (pos, { name = newname })
        end
    end
})
