-- occompass
-- Optionally place a compass and 24-hour clock on the screen.
-- A HUD version of my realcompass mod.
-- By David_G (kestral246@gmail.com)
-- 2019-12-31

local occompass = {}

local get_png   = function (basepng)
    return basepng
end

occompass.param_compass = {
    hud_elem_type="image", text="",
    position  = { x =  1 , y = 0 } , scale  = { x =  1 , y = 1 } ,
    alignment = { x = -1 , y = 1 } , offset = { x = -8 , y = 4 } ,
} ,

minetest.register_on_joinplayer(function(player)
    local pname = player:get_player_name()

    occompass [pname] = {
        id_compass = player:hud_add (occompass.param_compass) ,
        last_image = -1 ,
        state      =  1 ,
        counter    =  0 ,
    }
end)

minetest.register_on_leaveplayer (function (player)
    if player == nil then return end
    local pname = player:get_player_name()
    if pname  == nil then return end

    if  occompass [pname] then
        occompass [pname] = nil
    end
end)

local png_cache = {}

minetest.register_globalstep (function (dtime)

    local players  = minetest.get_connected_players()

    for i,player in ipairs (players) do

        local pname = player:get_player_name()
        if pname == nil or occompass [pname] == nil then return end

        local dir            = player:get_look_horizontal()
        local angle_relative = 360 - math.deg (dir)

        if    angle_relative <    0 then angle_relative = 0 end
        if    angle_relative >= 360 then angle_relative = 0 end

        local image_compass  = math.floor ((angle_relative / 22.5) + 0.5) % 16

        if  occompass [pname].last_image ~= image_compass then

            local png = png_cache [image_compass]
            if  png == nil then
                png =  get_png ("occompass_" .. image_compass .. ".png")
                        png_cache [image_compass] = png
            end

            local rc = player:hud_change (occompass [pname].id_compass,
                "text", png)

            if rc == 1 then
                occompass [pname].last_image = image_compass
            end
        end
    end
end)
