--[[
More Blocks: node definitions

Copyright (c) 2011-2017 Hugo Locurcio and contributors.
Licensed under the zlib license. See LICENSE.md for more information.
--]]

local S = function (x) return x end

local sound_dirt   = default.sound_dirt()
local sound_wood   = default.sound_wood()
local sound_stone  = default.sound_stone()
local sound_glass  = default.sound_glass()
local sound_leaves = default.sound_leaves()

-- Don't break on 0.4.14 and earlier.
local sound_metal = (default.sound_metal_defaults
        and default.sound_metal() or sound_stone)

local function tile_tiles(name)
    local tex = "moreblocks_" ..name.. ".png"
    return {tex, tex, tex, tex, tex.. "^[transformR90", tex.. "^[transformR90"}
end

local nodes = {
    ["wood_tile"] = {
        description = S("Wooden Tile"),
        groups = {snappy = 1, choppy = 2, oddly_breakable_by_hand = 2, flammable = 3} ,
        tiles = {"default_wood.png^moreblocks_wood_tile.png",
        "default_wood.png^moreblocks_wood_tile.png",
        "default_wood.png^moreblocks_wood_tile.png",
        "default_wood.png^moreblocks_wood_tile.png",
        "default_wood.png^moreblocks_wood_tile.png^[transformR90",
        "default_wood.png^moreblocks_wood_tile.png^[transformR90"} ,
        sounds = sound_wood,
    } ,
    ["wood_tile_flipped"] = {
        description = S("Wooden Tile"),
        groups = {wood = 1, snappy = 1, choppy = 2, oddly_breakable_by_hand = 2, flammable = 3} ,
        tiles = {"default_wood.png^moreblocks_wood_tile.png^[transformR90",
        "default_wood.png^moreblocks_wood_tile.png^[transformR90",
        "default_wood.png^moreblocks_wood_tile.png^[transformR90",
        "default_wood.png^moreblocks_wood_tile.png^[transformR90",
        "default_wood.png^moreblocks_wood_tile.png^[transformR180",
        "default_wood.png^moreblocks_wood_tile.png^[transformR180"} ,
        sounds = sound_wood,
        no_stairs = true,
    } ,
    ["wood_tile_center"] = {
        description = S("Centered Wooden Tile"),
        groups = {wood = 1, snappy = 1, choppy = 2, oddly_breakable_by_hand = 2, flammable = 3} ,
        tiles = {"default_wood.png^moreblocks_wood_tile_center.png"} ,
        sounds = sound_wood,
    } ,
    ["wood_tile_full"] = {
        description = S("Full Wooden Tile"),
        groups = {wood = 1, snappy = 1, choppy = 2, oddly_breakable_by_hand = 2, flammable = 3} ,
        tiles = tile_tiles("wood_tile_full"),
        sounds = sound_wood,
    } ,
    ["wood_tile_up"] = {
        description = S("Upwards Wooden Tile"),
        groups = {wood = 1, snappy = 1, choppy = 2, oddly_breakable_by_hand = 2, flammable = 3} ,
        tiles = {"default_wood.png^moreblocks_wood_tile_up.png"} ,
        sounds = sound_wood,
        no_stairs = true,
    } ,
    ["wood_tile_down"] = {
        description = S("Downwards Wooden Tile"),
        groups = {wood = 1, snappy = 1, choppy = 2, oddly_breakable_by_hand = 2, flammable = 3} ,
        tiles = {"default_wood.png^[transformR180^moreblocks_wood_tile_up.png^[transformR180"} ,
        sounds = sound_wood,
        no_stairs = true,
    } ,
    ["wood_tile_left"] = {
        description = S("Leftwards Wooden Tile"),
        groups = {wood = 1, snappy = 1, choppy = 2, oddly_breakable_by_hand = 2, flammable = 3} ,
        tiles = {"default_wood.png^[transformR270^moreblocks_wood_tile_up.png^[transformR270"} ,
        sounds = sound_wood,
        no_stairs = true,
    } ,
    ["wood_tile_right"] = {
        description = S("Rightwards Wooden Tile"),
        groups = {wood = 1, snappy = 1, choppy = 2, oddly_breakable_by_hand = 2, flammable = 3} ,
        tiles = {"default_wood.png^[transformR90^moreblocks_wood_tile_up.png^[transformR90"} ,
        sounds = sound_wood,
        no_stairs = true,
    } ,
    ["circle_stone_bricks"] = {
        description = S("Circle Stone Bricks"),
        groups = {cracky = 3} ,
        sounds = sound_stone,
    } ,
    ["grey_bricks"] = {
        description = S("Stone Bricks"),
        groups = {cracky = 3} ,
        sounds = sound_stone,
    } ,
    ["coal_stone_bricks"] = {
        description = S("Coal Stone Bricks"),
        groups = {cracky = 3} ,
        sounds = sound_stone,
    } ,
    ["iron_stone_bricks"] = {
        description = S("Iron Stone Bricks"),
        groups = {cracky = 3} ,
        sounds = sound_stone,
    } ,
    ["stone_tile"] = {
        description = S("Stone Tile"),
        groups = {cracky = 3} ,
        sounds = sound_stone,
    } ,
    ["split_stone_tile"] = {
        description = S("Split Stone Tile"),
        tiles = {"moreblocks_split_stone_tile_top.png",
            "moreblocks_split_stone_tile.png"} ,
        groups = {cracky = 3} ,
        sounds = sound_stone,
    } ,
    ["split_stone_tile_alt"] = {
        description = S("Split Stone Tile"),
        groups = {cracky = 3} ,
        sounds = sound_stone,
    } ,
    ["checker_stone_tile"] = {
        description = S("Checker Stone Tile"),
        groups = {stone = 1, cracky = 3} ,
        is_ground_content = false,
        sounds = sound_stone,
    } ,
    ["tar"] = {
        description = S("Tar"),
        groups = {cracky = 2, tar_block = 1} ,
        sounds = sound_stone,
    } ,
    ["dirt_compressed"] = {
        description = S("Compressed Dirt"),
        groups = {crumbly=2} ,
        is_ground_content = false,
        sounds = sound_dirt,
    } ,
    ["cobble_compressed"] = {
        description = S("Compressed Cobblestone"),
        groups = {cracky = 1} ,
        sounds = sound_stone,
    } ,
    ["plankstone"] = {
        description = S("Plankstone"),
        groups = {cracky = 3} ,
        tiles = tile_tiles("plankstone"),
        sounds = sound_stone,
    } ,
    ["iron_glass"] = {
        description = S("Iron Glass"),
        drawtype = "glasslike_framed_optional",
        tiles = {"moreblocks_iron_glass.png"} ,
        paramtype = "light",
        sunlight_propagates = true,
        groups = {snappy = 2, cracky = 3, oddly_breakable_by_hand = 3} ,
        sounds = sound_glass,
    } ,
    ["coal_glass"] = {
        description = S("Coal Glass"),
        drawtype = "glasslike_framed_optional",
        tiles = {"moreblocks_coal_glass.png"} ,
        paramtype = "light",
        sunlight_propagates = true,
        groups = {snappy = 2, cracky = 3, oddly_breakable_by_hand = 3} ,
        sounds = sound_glass,
    } ,
    ["clean_glass"] = {
        description = S("Clean Glass"),
        drawtype = "glasslike_framed_optional",
        tiles = {"moreblocks_clean_glass.png"} ,
        paramtype = "light",
        sunlight_propagates = true,
        groups = {snappy = 2, cracky = 3, oddly_breakable_by_hand = 3} ,
        sounds = sound_glass,
    } ,
    ["cactus_brick"] = {
        description = S("Cactus Brick"),
        groups = {cracky = 3} ,
        sounds = sound_stone,
    } ,
    ["cactus_checker"] = {
        description = S("Cactus Checker"),
        groups = {cracky = 3} ,
        tiles = {"default_stone.png^moreblocks_cactus_checker.png",
        "default_stone.png^moreblocks_cactus_checker.png",
        "default_stone.png^moreblocks_cactus_checker.png",
        "default_stone.png^moreblocks_cactus_checker.png",
        "default_stone.png^moreblocks_cactus_checker.png^[transformR90",
        "default_stone.png^moreblocks_cactus_checker.png^[transformR90"} ,
        sounds = sound_stone,
    } ,
    ["empty_bookshelf"] = {
        description = S("Empty Bookshelf"),
        tiles = {"default_wood.png", "default_wood.png",
            "moreblocks_empty_bookshelf.png"} ,
        groups = {snappy = 2, choppy = 3, oddly_breakable_by_hand = 2, flammable = 3} ,
        sounds = sound_wood,
        furnace_burntime = 15,
        no_stairs = true,
    } ,
    ["empty_shelf"] = {
        description = S("Empty Shelf"),
        paramtype2 = "facedir",
        tiles = {"default_wood.png", "default_wood.png", "default_wood.png",
        "default_wood.png", "moreblocks_empty_shelf.png", "moreblocks_empty_shelf.png"} ,
        groups = {choppy = 3, oddly_breakable_by_hand = 2, flammable = 3} ,
        is_ground_content = false,
        sounds = sound_wood,
        furnace_burntime = 15,
        no_stairs = true,
    } ,
    ["coal_stone"] = {
        description = S("Coal Stone"),
        groups = {cracky = 3} ,
        sounds = sound_stone,
    } ,
    ["iron_stone"] = {
        description = S("Iron Stone"),
        groups = {cracky = 3} ,
        sounds = sound_stone,
    } ,
    ["coal_checker"] = {
        description = S("Coal Checker"),
        tiles = {"default_stone.png^moreblocks_coal_checker.png",
        "default_stone.png^moreblocks_coal_checker.png",
        "default_stone.png^moreblocks_coal_checker.png",
        "default_stone.png^moreblocks_coal_checker.png",
        "default_stone.png^moreblocks_coal_checker.png^[transformR90",
        "default_stone.png^moreblocks_coal_checker.png^[transformR90"} ,
        groups = {cracky = 3} ,
        sounds = sound_stone,
    } ,
    ["iron_checker"] = {
        description = S("Iron Checker"),
        tiles = {"default_stone.png^moreblocks_iron_checker.png",
        "default_stone.png^moreblocks_iron_checker.png",
        "default_stone.png^moreblocks_iron_checker.png",
        "default_stone.png^moreblocks_iron_checker.png",
        "default_stone.png^moreblocks_iron_checker.png^[transformR90",
        "default_stone.png^moreblocks_iron_checker.png^[transformR90"} ,
        groups = {cracky = 3} ,
        sounds = sound_stone,
    } ,
    ["all_faces_tree"] = {
        description = S("All-faces Tree"),
        tiles = {"default_tree_top.png"} ,
        groups = {tree = 1,snappy = 1, choppy = 2, oddly_breakable_by_hand = 1, flammable = 2} ,
        sounds = sound_wood,
        furnace_burntime = 30,
    } ,
    ["glow_glass"] = {
        description = S("Glow Glass"),
        drawtype = "glasslike_framed_optional",
        tiles = {"moreblocks_glow_glass.png"} ,
        paramtype = "light",
        sunlight_propagates = true,
        light_source = 11,
        groups = {snappy = 2, cracky = 3, oddly_breakable_by_hand = 3} ,
        sounds = sound_glass,
    } ,
    ["super_glow_glass"] = {
        description = S("Super Glow Glass"),
        drawtype = "glasslike_framed_optional",
        tiles = {"moreblocks_super_glow_glass.png"} ,
        paramtype = "light",
        sunlight_propagates = true,
        light_source = 14,
        groups = {snappy = 2, cracky = 3, oddly_breakable_by_hand = 3} ,
        sounds = sound_glass,
    } ,
    ["rope"] = {
        description = S("Rope"),
        drawtype = "signlike",
        inventory_image = "moreblocks_rope.png",
        wield_image = "moreblocks_rope.png",
        paramtype = "light",
        sunlight_propagates = true,
        paramtype2 = "wallmounted",
        walkable = false,
        climbable = true,
        selection_box = {type = "wallmounted",} ,
        groups = {snappy = 3, flammable = 2} ,
        sounds = sound_leaves,
        no_stairs = true,
    } ,
    ["copperpatina"] = {
        description = S("Copper Patina Block"),
        groups = {cracky = 1, level = 2} ,
        sounds = sound_metal,
    } ,
}

for name, def in pairs (nodes) do
    def.tiles = def.tiles or {"moreblocks_" ..name.. ".png"}
    ocutil.safe_register_node ("moreblocks:" ..name , def)
    ocutil.safe_register_alias (name, "moreblocks:" ..name)
end

-- Items

minetest.register_craftitem (":moreblocks:sweeper", {
    description     = S("Sweeper") ,
    inventory_image = "moreblocks_sweeper.png" ,
})

-- ===================================================================

ocutil.safe_register_node ("moreblocks:horizontal_tree", {
    description      = "Horizontal Tree" ,
    furnace_burntime = 30                ,
    paramtype2       = "facedir"         ,

    tiles = {
        "default_tree.png"               ,
        "default_tree.png"               ,
        "default_tree.png^[transformR90" ,
        "default_tree.png^[transformR90" ,
        "default_tree_top.png"           ,
        "default_tree_top.png"           ,
    } ,

    groups = {
        tree=1, snappy=1, choppy=2,
        oddly_breakable_by_hand=1 ,flammable=2
    } ,

    sounds = default.sound_wood() ,
})
